@@ -36,10 +36,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-
-#if defined(HAVE_POSIX_MEMALIGN)
 #include <stdlib.h>
-#endif
 
 /*
  * Optional library and BFT headers
@@ -132,10 +129,6 @@ struct _bft_mem_block_t {
  * Local macro definitions
  *-----------------------------------------------------------------------------*/
 
-#define BFT_KILO_BYTE        1024   /* 2^10 : bytes to kilobytes */
-#define BFT_MEGA_BYTE     1048576   /* 2^20 : bytes to Megabytes */
-#define BFT_GIGA_BYTE  1073741824   /* 2^30 : bytes to Gigabytes */
-
 /*-----------------------------------------------------------------------------
  * Local function prototypes
  *-----------------------------------------------------------------------------*/
@@ -172,12 +165,15 @@ static FILE *_bft_mem_global_file = NULL;
 
 static struct _bft_mem_block_t  *_bft_mem_global_block_array = NULL;
 
-static unsigned long  _bft_mem_global_block_nbr   = 0 ;
-static unsigned long  _bft_mem_global_block_max   = 512 ;
+static unsigned long  _bft_mem_global_block_nbr = 0 ;
+static unsigned long  _bft_mem_global_block_max = 512 ;
 
-static long  _bft_mem_global_alloc_cur[2] = {0, 0};
-static long  _bft_mem_global_alloc_max[2] = {0, 0};
-static long  _bft_mem_global_alloc_tot[2] = {0, 0};
+static size_t  _bft_mem_global_alloc_cur = 0;
+static size_t  _bft_mem_global_alloc_max = 0;
+
+static size_t  _bft_mem_global_n_allocs = 0;
+static size_t  _bft_mem_global_n_reallocs = 0;
+static size_t  _bft_mem_global_n_frees = 0;
 
 static bft_error_handler_t  *_bft_mem_error_handler
                               = (_bft_mem_error_handler_default);
@@ -215,55 +211,6 @@ _bft_mem_basename(const char  *file_name)
   return (file_name + i);
 }
 
-/*
- * Add an integer to an array representing a long integer.
- *
- * parameters:
- *   counter:   <-> portable long integer representation.
- *   increment: <-- integer to add (or substract).
- */
-
-static void
-_bft_mem_counter_incr(long counter[2],
-                      long increment)
-{
-
-  counter[0] += increment;
-
-  if (increment >= 0) {
-    while (counter[0] > BFT_GIGA_BYTE) {
-      counter[0] -= BFT_GIGA_BYTE;
-      counter[1] += 1;
-    }
-  }
-  else {
-    while (counter[0] < 0) {
-      counter[0] += BFT_GIGA_BYTE;
-      counter[1] -= 1;
-    }
-  }
-}
-
-/*
- * Affect the maximum values of two counters to the first one.
- *
- * parameters:
- *   counter:     <-> counter to update.
- *   counter_ref: <-- counter to compare with counter.
- */
-
-static void
-_bft_mem_counter_max(long       counter[2],
-                     const long counter_ref[2]
-)
-{
-  if (   (counter_ref[0] >  counter[0])
-      && (counter_ref[1] >= counter[1])) {
-    counter[0] = counter_ref[0];
-    counter[1] = counter_ref[1];
-  }
-}
-
 /*
  * Determines values associated with an array representing a
  * long integer
@@ -271,90 +218,30 @@ _bft_mem_counter_max(long       counter[2],
  * parameters:
  *   counter: <-- counter to update.
  *   value:   --> counter values in output unit [out].
- *   unit:    --> counter value unit : ' ', 'K', 'M', 'G', 'T', or 'P'
+ *   unit:    --> counter value unit : ' ', 'k', 'm', 'g', 't', or 'p'
  *                for bytes, Kilobytes, Megabytes, Gigabytes, Terabytes,
  *                or Petabytes [out].
  */
 
 static void
-_bft_mem_counter_val(const long    counter[2],
-                     long          value[2],
-                     char        * unit)
+_bft_mem_size_val(const size_t    counter,
+                  unsigned long   value[2],
+                  char           *unit)
 {
-  /* If the counter is under 1 G */
+  int i;
+  size_t _counter[2];
+  const char units[] = {' ', 'k', 'm', 'g', 't', 'p', 'e'};
 
-  if (counter[1] == 0) {
-
-    if (counter[0] < BFT_KILO_BYTE) {
-      value[0] = counter[0];
-      value[1] = 0;
-      *unit    = ' ';
-    }
-    else if (counter[0] < BFT_MEGA_BYTE) {
-      value[0] = counter[0] / BFT_KILO_BYTE;
-      value[1] = (counter[0] % BFT_KILO_BYTE);
-      *unit    = 'k';
-    }
-    else {
-      value[0] = counter[0] / BFT_MEGA_BYTE;
-      value[1] = (counter[0] % BFT_MEGA_BYTE) / BFT_KILO_BYTE;
-      *unit    = 'm';
-    }
+  for (i = 0, _counter[0] = counter, _counter[1] = 0;
+       _counter[0] >= 1024 && i < 6;
+       i++) {
+    _counter[1] = _counter[0] % 1024;
+    _counter[0] /= 1024;
   }
 
-  /* If the counter is over 1 G but under 1 T*/
-
-  else if (counter[1] < BFT_KILO_BYTE) {
-      value[0] = counter[1];
-      value[1] = (counter[0] / BFT_MEGA_BYTE);
-      *unit    = 'g';
-  }
-
-  /* If the counter is over 1 T */
-
-  else {
-
-    if (counter[1] < BFT_MEGA_BYTE) {
-      value[0] = counter[1] / BFT_KILO_BYTE;
-      value[1] = (counter[1] % BFT_KILO_BYTE);
-      *unit    = 't';
-    }
-    else {
-      value[0] = counter[0] / BFT_MEGA_BYTE;
-      value[1] = (counter[0] % BFT_MEGA_BYTE) / BFT_KILO_BYTE;
-      *unit    = 'p';
-    }
-  }
-
-  /* We should not risk overflow of these values before some time
-     on 32 bit architectures, and even on 64 bit architectures,
-     we have an additional safety margin if "long" is a 64 bit
-     integer type. */
-}
-
-/*
- * Transform a counter value to kilo-bytes.
- *
- * parameters:
- *   counter: <-> portable long integer representation.
- *
- * return:
- *   size in kilo-bytes.
- */
-
-static size_t
-_bft_mem_counter_to_kb(const long  counter[2])
-{
-  size_t size;
-
-  /* If the counter is under 1 G */
-
-  size = counter[0] / BFT_KILO_BYTE;
-
-  if (counter[1] > 0)
-    size += counter[1] * (BFT_GIGA_BYTE/BFT_KILO_BYTE);
-
-  return size;
+  value[0] = _counter[0];
+  value[1] = _counter[1];
+  *unit = units[i];
 }
 
 /*
@@ -363,9 +250,9 @@ _bft_mem_counter_to_kb(const long  counter[2])
 
 static void _bft_mem_summary(FILE  *f)
 {
-  char   unit;
-  long   value[2];
-  int    track_mode;
+  char unit;
+  unsigned long value[2];
+  int track_mode;
   size_t mem_usage;
   static const char *track_state[2] = {N_("off"),
                                        N_("on")};
@@ -379,20 +266,24 @@ static void _bft_mem_summary(FILE  *f)
 
   /* Available memory usage information */
 
-  _bft_mem_counter_val(_bft_mem_global_alloc_cur, value, &unit);
+  _bft_mem_size_val(_bft_mem_global_alloc_cur, value, &unit);
   fprintf(f,
           _("Theoretical current allocated memory:   %8ld.%ld %cB\n"),
           value[0], value[1], unit);
 
-  _bft_mem_counter_val(_bft_mem_global_alloc_max, value, &unit);
+  _bft_mem_size_val(_bft_mem_global_alloc_max, value, &unit);
   fprintf(f,
           _("Theoretical maximum allocated memory:   %8ld.%ld %cB\n"),
           value[0], value[1], unit);
 
-  _bft_mem_counter_val(_bft_mem_global_alloc_tot, value, &unit);
   fprintf(f,
-          _("Theoretical cumulated allocated memory: %8ld.%ld %cB\n\n"),
-          value[0], value[1], unit);
+          _("\n"
+            "Number of allocations:   %lu\n"
+            "          reallocations: %lu\n"
+            "          frees:         %lu\n\n"),
+          (unsigned long)_bft_mem_global_n_allocs,
+          (unsigned long)_bft_mem_global_n_reallocs,
+          (unsigned long)_bft_mem_global_n_frees);
 
   if (bft_mem_usage_initialized() == 1) {
 
@@ -825,12 +716,12 @@ void bft_mem_end(void)
   _bft_mem_global_block_nbr   = 0 ;
   _bft_mem_global_block_max   = 512 ;
 
-  _bft_mem_global_alloc_cur[0] = 0;
-  _bft_mem_global_alloc_cur[1] = 0;
-  _bft_mem_global_alloc_max[0] = 0;
-  _bft_mem_global_alloc_max[1] = 0;
-  _bft_mem_global_alloc_tot[0] = 0;
-  _bft_mem_global_alloc_tot[1] = 0;
+  _bft_mem_global_alloc_cur = 0;
+  _bft_mem_global_alloc_max = 0;
+
+  _bft_mem_global_n_allocs = 0;
+  _bft_mem_global_n_reallocs = 0;
+  _bft_mem_global_n_frees = 0;
 
 }
 
@@ -890,10 +781,10 @@ bft_mem_malloc(size_t       ni,
 
   /* Memory allocation counting */
 
-  _bft_mem_counter_incr(_bft_mem_global_alloc_cur, alloc_size);
-  _bft_mem_counter_incr(_bft_mem_global_alloc_tot, alloc_size);
+  _bft_mem_global_alloc_cur += alloc_size;
 
-  _bft_mem_counter_max(_bft_mem_global_alloc_max, _bft_mem_global_alloc_cur);
+  if (_bft_mem_global_alloc_max < _bft_mem_global_alloc_cur)
+    _bft_mem_global_alloc_max = _bft_mem_global_alloc_cur;
 
   if (_bft_mem_global_file != NULL) {
     fprintf(_bft_mem_global_file, "\n  alloc: %-27s:%6d : %-39s: %9lu",
@@ -901,14 +792,15 @@ bft_mem_malloc(size_t       ni,
             var_name, (unsigned long)alloc_size);
     fprintf(_bft_mem_global_file, " : (+%9lu) : %12lu : [%10p]",
             (unsigned long)alloc_size,
-            (unsigned long)(  _bft_mem_global_alloc_cur[0]
-                            + _bft_mem_global_alloc_cur[1]*BFT_GIGA_BYTE),
+            (unsigned long)_bft_mem_global_alloc_cur,
             p_loc);
     fflush(_bft_mem_global_file);
   }
 
   _bft_mem_block_malloc(p_loc, alloc_size);
 
+  _bft_mem_global_n_allocs += 1;
+
   /* Return pointer to allocated memory */
 
   return p_loc;
@@ -994,11 +886,11 @@ bft_mem_realloc(void        *ptr,
     else if (_bft_mem_global_initialized == 0)
       return p_loc;
 
-    _bft_mem_counter_incr(_bft_mem_global_alloc_cur, size_diff);
+    _bft_mem_global_alloc_cur += size_diff;
 
     if (size_diff > 0) {
-      _bft_mem_counter_incr(_bft_mem_global_alloc_tot, size_diff);
-      _bft_mem_counter_max(_bft_mem_global_alloc_max, _bft_mem_global_alloc_cur);
+      if (_bft_mem_global_alloc_max < _bft_mem_global_alloc_cur)
+        _bft_mem_global_alloc_max = _bft_mem_global_alloc_cur;
     }
 
     if (_bft_mem_global_file != NULL) {
@@ -1009,14 +901,15 @@ bft_mem_realloc(void        *ptr,
       fprintf(_bft_mem_global_file, " : (%c%9lu) : %12lu : [%10p]",
               sgn,
               (unsigned long) ((size_diff > 0) ? size_diff : -size_diff),
-              (unsigned long)(  _bft_mem_global_alloc_cur[0]
-                              + _bft_mem_global_alloc_cur[1]*BFT_GIGA_BYTE),
+              (unsigned long)_bft_mem_global_alloc_cur,
               p_loc);
       fflush(_bft_mem_global_file);
     }
 
     _bft_mem_block_realloc(ptr, p_loc, new_size);
 
+    _bft_mem_global_n_reallocs += 1;
+
     return p_loc;
   }
 
@@ -1058,7 +951,7 @@ bft_mem_free(void        *ptr,
 
     size_info = _bft_mem_block_size(ptr);
 
-    _bft_mem_counter_incr(_bft_mem_global_alloc_cur, -size_info);
+    _bft_mem_global_alloc_cur -= size_info;
 
     if (_bft_mem_global_file != NULL) {
       fprintf(_bft_mem_global_file,"\n   free: %-27s:%6d : %-39s: %9lu",
@@ -1066,14 +959,14 @@ bft_mem_free(void        *ptr,
               var_name, (unsigned long)size_info);
       fprintf(_bft_mem_global_file, " : (-%9lu) : %12lu : [%10p]",
               (unsigned long)size_info,
-              (unsigned long)(  _bft_mem_global_alloc_cur[0]
-                              + _bft_mem_global_alloc_cur[1]*BFT_GIGA_BYTE),
+              (unsigned long)_bft_mem_global_alloc_cur,
               ptr);
       fflush(_bft_mem_global_file);
     }
 
     _bft_mem_block_free(ptr);
 
+    _bft_mem_global_n_frees += 1;
   }
 
   free(ptr);
@@ -1143,10 +1036,10 @@ bft_mem_memalign(size_t       alignment,
 
   /* Memory allocation counting */
 
-  _bft_mem_counter_incr(_bft_mem_global_alloc_cur, alloc_size);
-  _bft_mem_counter_incr(_bft_mem_global_alloc_tot, alloc_size);
+  _bft_mem_global_alloc_cur += alloc_size;
 
-  _bft_mem_counter_max(_bft_mem_global_alloc_max, _bft_mem_global_alloc_cur);
+  if (_bft_mem_global_alloc_max < _bft_mem_global_alloc_cur)
+    _bft_mem_global_alloc_max = _bft_mem_global_alloc_cur;
 
   if (_bft_mem_global_file != NULL) {
     fprintf(_bft_mem_global_file, "\n  alloc: %-27s:%6d : %-39s: %9lu",
@@ -1154,14 +1047,15 @@ bft_mem_memalign(size_t       alignment,
             var_name, (unsigned long)alloc_size);
     fprintf(_bft_mem_global_file, " : (+%9lu) : %12lu : [%10p]",
             (unsigned long)alloc_size,
-            (unsigned long)(  _bft_mem_global_alloc_cur[0]
-                            + _bft_mem_global_alloc_cur[1]*BFT_GIGA_BYTE),
+            (unsigned long)_bft_mem_global_alloc_cur,
             p_loc);
     fflush(_bft_mem_global_file);
   }
 
   _bft_mem_block_malloc(p_loc, alloc_size);
 
+  _bft_mem_global_n_allocs += 1;
+
   /* Return pointer to allocated memory */
 
   return p_loc;
@@ -1183,7 +1077,7 @@ bft_mem_memalign(size_t       alignment,
 size_t
 bft_mem_size_current(void)
 {
-  return _bft_mem_counter_to_kb(_bft_mem_global_alloc_cur);
+  return (_bft_mem_global_alloc_cur / 1024);
 }
 
 /*!
@@ -1195,7 +1089,7 @@ bft_mem_size_current(void)
 size_t
 bft_mem_size_max(void)
 {
-  return _bft_mem_counter_to_kb(_bft_mem_global_alloc_max);
+  return (_bft_mem_global_alloc_max / 1024);
 }
 
 /*!
